'use strict';

var RequireContextDependency = require('./RequireContextDependency');
var RequireDynamicContextDependency = require('./RequireDynamicContextDependency');

var ContextDependencyHelpers = exports;

/**
 * Escapes regular expression metacharacters
 * @param {string} str String to quote
 * @return {string} Escaped string
 */
function quotemeta(str) {
  return str.replace(/[-[\]\\/{}()*+?.^$|]/g, '\\$&');
}

ContextDependencyHelpers.create = function (range, param, expr, options, contextOptions) {
  if (param.isWrapped() && param.prefix && param.prefix.isString()) {
    var prefix = param.prefix.string;
    var prefixRange = param.prefix.range;
    var valueRange = [prefixRange ? prefixRange[1] : param.range[0], param.range[1]];
    var idx = prefix.lastIndexOf('/');
    var context = '.';
    if (idx >= 0) {
      context = prefix.substr(0, idx);
      prefix = `.${prefix.substr(idx)}`;
    }
    var regExp = new RegExp(`^${quotemeta(prefix)}.*$`);
    var _dep = new RequireContextDependency(Object.assign({
      request: context,
      recursive: options.wrappedContextRecursive,
      regExp,
      mode: 'sync'
    }, contextOptions), range, valueRange);
    _dep.loc = expr.loc;
    _dep.prepend = param.prefix && param.prefix.isString() ? prefix : null;
    _dep.critical = options.wrappedContextCritical && 'a part of the request of a dependency is an expression';
    return _dep;
  }

  // if there is no prefix we use the custom dynamic module which contains all ui5 resources
  var dep = new RequireDynamicContextDependency(range, param.range);

  return dep;
};

ContextDependencyHelpers.createView = function (Dep, range, param, expr, options, contextOptions) {
  var prefix = 'sap/ui/core/mvc/';
  var regExp = new RegExp(`View.js$`);
  var dep = new Dep(Object.assign({
    request: prefix,
    recursive: options.wrappedContextRecursive,
    regExp,
    mode: 'weak'
  }, contextOptions), range, param.range);
  dep.loc = expr.loc;
  dep.prepend = null;
  dep.critical = options.wrappedContextCritical && 'a part of the request of a dependency is an expression';
  dep.replaces = [{
    range: [dep.valueRange[0] - 1, dep.valueRange[0] - 1],
    value: 'new ('
  }, {
    range: [dep.valueRange[1], dep.valueRange[1]],
    value: '.replace("sap/ui/core/mvc", ".") + ".js")'
  }, {
    range: [dep.range[1], dep.range[1]],
    value: '(oView)'
  }];
  return dep;
};