'use strict';

var cssnano = require('cssnano');
var lessOpenUI5 = require('less-openui5');

var _require = require('webpack-sources'),
    OriginalSource = _require.OriginalSource;

var NullFactory = require('webpack/lib/NullFactory');
var LocalModuleDependency = require('webpack/lib/dependencies/LocalModuleDependency');
var ConstDependency = require('webpack/lib/dependencies/ConstDependency');
var ContextElementDependency = require('webpack/lib/dependencies/ContextElementDependency');
var DefineDependency = require('./DefineDependency');
var DefineDependencyParserPlugin = require('./DefineDependencyParserPlugin');
var DynamicContextModuleFactory = require('./DynamicContextModuleFactory');
var OpenUI5ViewDependency = require('./ViewDependency');
var RequireDependencyParserPlugin = require('./RequireDependencyParserPlugin');
var RequireDependency = require('./RequireDependency');
var RequireItemDependency = require('./RequireItemDependency');
var RequireContextDependency = require('./RequireContextDependency');
var RequireDynamicContextDependency = require('./RequireDynamicContextDependency');
var ResourceDependencyParserPlugin = require('./ResourceDependencyParserPlugin');
var ResourceDependency = require('./ResourceDependency');
var ResourceModuleFactory = require('./ResourceModuleFactory');
var JQueryDependencyParserPlugin = require('./JQueryDependencyParserPlugin');
var JQueryItemDependency = require('./JQueryItemDependency');
var GlobalDependencyParserPlugin = require('./GlobalDependencyParserPlugin');
var ViewDependencyParserPlugin = require('./ViewDependencyParserPlugin');

class OpenUI5Plugin {
  constructor(options) {
    this.options = options || {};
  }

  apply(compiler) {
    var options = this.options;


    var resourceModuleFactory = new ResourceModuleFactory(compiler.resolverFactory);
    var dynamicContextModuleFactory = new DynamicContextModuleFactory(options.requireSync);

    compiler.hooks.compilation.tap('OpenUI5Plugin', function (compilation, _ref) {
      var normalModuleFactory = _ref.normalModuleFactory,
          contextModuleFactory = _ref.contextModuleFactory;

      compilation.dependencyFactories.set(RequireDependency, new NullFactory());
      compilation.dependencyTemplates.set(RequireDependency, new RequireDependency.Template());

      compilation.dependencyFactories.set(RequireItemDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(RequireItemDependency, new RequireItemDependency.Template());

      compilation.dependencyFactories.set(RequireContextDependency, contextModuleFactory);
      compilation.dependencyTemplates.set(RequireContextDependency, new RequireContextDependency.Template());

      compilation.dependencyFactories.set(RequireDynamicContextDependency, dynamicContextModuleFactory);
      compilation.dependencyTemplates.set(RequireDynamicContextDependency, new RequireDynamicContextDependency.Template());

      compilation.dependencyFactories.set(DefineDependency, new NullFactory());
      compilation.dependencyTemplates.set(DefineDependency, new DefineDependency.Template());

      compilation.dependencyFactories.set(JQueryItemDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(JQueryItemDependency, new JQueryItemDependency.Template());

      compilation.dependencyFactories.set(ResourceDependency, resourceModuleFactory);
      compilation.dependencyTemplates.set(ResourceDependency, new ResourceDependency.Template());

      compilation.dependencyFactories.set(OpenUI5ViewDependency, normalModuleFactory);
      compilation.dependencyTemplates.set(OpenUI5ViewDependency, new OpenUI5ViewDependency.Template());

      compilation.dependencyFactories.set(LocalModuleDependency, new NullFactory());
      compilation.dependencyTemplates.set(LocalModuleDependency, new LocalModuleDependency.Template());

      compilation.dependencyFactories.set(ConstDependency, new NullFactory());
      compilation.dependencyTemplates.set(ConstDependency, new ConstDependency.Template());

      compilation.dependencyFactories.set(ContextElementDependency, normalModuleFactory);
      compilation.dependencyFactories.set(ContextElementDependency, normalModuleFactory);

      normalModuleFactory.hooks.parser.for('javascript/auto').tap('OpenUI5Plugin', function (parser) {
        new RequireDependencyParserPlugin(options).apply(parser);
        new DefineDependencyParserPlugin(options).apply(parser);
        new JQueryDependencyParserPlugin().apply(parser);
        new GlobalDependencyParserPlugin().apply(parser);
        new ViewDependencyParserPlugin(options).apply(parser);
        new ResourceDependencyParserPlugin(options).apply(parser);
      });
    });

    if (options.theme) {
      compiler.hooks.thisCompilation.tap('OpenUI5Plugin', function (compilation) {
        compilation.hooks.additionalAssets.tapAsync('OpenUI5Plugin', function (callback) {
          var builder = new lessOpenUI5.Builder();
          var promises = [];

          var themes = options.theme;
          if (!Array.isArray(themes)) {
            themes = [themes];
          }

          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = themes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var theme = _step.value;

              var _loop = function _loop(lib) {
                var libPath = `${lib.replace(/\./g, '/')}/themes/${theme}`;
                var promise = builder.build({
                  lessInputPath: `${libPath}/library.source.less`,
                  rootPaths: options.rootPaths,
                  library: {
                    name: lib
                  }
                }).then(function (result) {
                  return cssnano.process(result.css, {
                    postcssNormalizeUrl: false,
                    reduceIdents: false
                  }).then(function (result) {
                    var file = `${libPath}/library.css`;
                    compilation.assets[file] = new OriginalSource(result.css, file);
                  });
                }).catch(callback);

                promises.push(promise);
              };

              var _iteratorNormalCompletion2 = true;
              var _didIteratorError2 = false;
              var _iteratorError2 = undefined;

              try {
                for (var _iterator2 = options.libs[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                  var lib = _step2.value;

                  _loop(lib);
                }
              } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion2 && _iterator2.return) {
                    _iterator2.return();
                  }
                } finally {
                  if (_didIteratorError2) {
                    throw _iteratorError2;
                  }
                }
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          Promise.all(promises).then(function () {
            callback();
          }).catch(callback);
        });
      });
    }
  }
}
module.exports = OpenUI5Plugin;