'use strict';

var OpenUI5ResourceDependency = require('./OpenUI5ResourceDependency');

class OpenUI5ResourceDependencyParserPlugin {
  constructor(options) {
    this.options = options;
  }

  apply(parser) {
    var options = this.options || {};

    parser.hooks.call.for('jQuery.sap.loadResource').tap('OpenUI5Plugin', function (expr) {
      var modulePath = options.modulePath || '';
      var resources = options.resources || {};
      var extensions = resources.extensions || ['properties', 'xml'];
      var translations = options.translations || [];
      var libraries = options.libs || [];
      var arg = expr.arguments[0];

      var param = void 0;
      var foe = void 0;
      if (arg.type === 'ObjectExpression') {
        var property = arg.properties.find(function (prop) {
          if (prop.key && prop.key.type === 'Identifier' && prop.key.name === 'url') {
            return true;
          }
          return false;
        });

        if (!property) {
          throw new Error('Property url missing in object for jQuery.sap.loadResource');
        }

        var failOnError = arg.properties.find(function (prop) {
          if (prop.key && prop.key.type === 'Identifier' && prop.key.name === 'failOnError') {
            return true;
          }
          return false;
        });

        param = parser.evaluateExpression(property.value);
        if (failOnError) {
          foe = parser.evaluateExpression(failOnError.value);
        }
      } else {
        param = parser.evaluateExpression(arg);
      }

      var dep = new OpenUI5ResourceDependency(parser.state.options.context, modulePath, extensions, libraries, translations, foe ? foe.bool : true, expr.range, param.range);
      dep.loc = expr.loc;
      parser.state.current.addDependency(dep);
      return true;
    });
  }
}
module.exports = OpenUI5ResourceDependencyParserPlugin;