'use strict';

var ContextDependency = require('webpack/lib/dependencies/ContextDependency');
var ContextDependencyTemplateAsRequireCall = require('webpack/lib/dependencies/ContextDependencyTemplateAsRequireCall');

class OpenUI5ViewContextDependency extends ContextDependency {
  constructor(options, range, valueRange) {
    super(options);
    this.range = range;
    this.valueRange = valueRange;
  }

  get type() {
    return 'openui5 view context';
  }
}

OpenUI5ViewContextDependency.Template = class OpenUI5ViewContextDependencyTemplateAsRequireCall {
  apply(dep, source, runtime) {
    var moduleExports = runtime.moduleExports({
      module: dep.module,
      request: dep.request
    });

    if (dep.module) {
      if (dep.valueRange) {
        if (Array.isArray(dep.replaces)) {
          for (var i = 0; i < dep.replaces.length; i++) {
            var rep = dep.replaces[i];
            source.replace(rep.range[0], rep.range[1] - 1, rep.value);
          }
        }
        source.replace(dep.valueRange[1], dep.range[1] - 1, ")");
        source.replace(dep.range[0], dep.valueRange[0] - 1, `${moduleExports}(${typeof dep.prepend === "string" ? JSON.stringify(dep.prepend) : ""}`);
      } else {
        source.replace(dep.range[0], dep.range[1] - 1, moduleExports);
      }
    } else {
      source.replace(dep.range[0], dep.range[1] - 1, moduleExports);
    }
  }
};

module.exports = OpenUI5ViewContextDependency;