'use strict';

var async = require('async');
var Tapable = require('tapable').Tapable;
var AsyncSeriesWaterfallHook = require('tapable').AsyncSeriesWaterfallHook;
var ResourceModule = require('./ResourceModule');

class ResourceModuleFactory extends Tapable {
  constructor(resolverFactory) {
    super();
    this.hooks = {
      beforeResolve: new AsyncSeriesWaterfallHook(['data']),
      afterResolve: new AsyncSeriesWaterfallHook(['data'])
    };
    this.resolverFactory = resolverFactory;
  }

  create(data, callback) {
    var _this = this;

    var dependencies = data.dependencies;
    var dependency = dependencies[0];
    var resolveOptions = data.resolveOptions;

    this.hooks.beforeResolve.callAsync({
      context: dependency.context,
      modulePath: dependency.modulePath,
      extensions: dependency.extensions,
      libraries: dependency.libraries,
      translations: dependency.translations,
      locales: dependency.locales ? dependency.locales : dependency.translations,
      failOnError: dependency.failOnError
    }, function (err, result) {
      var context = result.context;
      var modulePath = result.modulePath;
      var extensions = result.extensions;
      var libraries = result.libraries;
      var translations = result.translations;
      var locales = result.locales;
      var failOnError = result.failOnError;

      var messagebundles = ['messagebundle.properties'];
      translations.forEach(function (translation) {
        messagebundles.push(`messagebundle_${translation}.properties`);
      });

      var resources = [];
      libraries.forEach(function (library) {
        var libSlash = library.replace(/\./g, '/');
        messagebundles.forEach(function (messagebundle) {
          resources.push(`${libSlash}/${messagebundle}`);
        });
      });
      locales.forEach(function (translation) {
        resources.push(`sap/ui/core/cldr/${translation}.json`);
      });

      var normalResolver = _this.resolverFactory.get('normal', resolveOptions || {});

      async.map(resources, function (resource, callback) {
        normalResolver.resolve({}, dependency.context, resource, {}, function (err) {
          if (err) {
            callback();
            return;
          }
          callback(null, resource);
        });
      }, function (err, result) {
        if (err) {
          callback(err);
          return;
        }

        var resources = result.filter(function (resource) {
          return resource;
        });

        _this.hooks.afterResolve.callAsync({
          context,
          modulePath,
          extensions,
          resources,
          failOnError
        }, function (err, result) {
          if (err) return callback(err);

          // Ignored
          if (!result) return callback();

          return callback(null, new ResourceModule(result));
        });
      });
    });
  }
}

module.exports = ResourceModuleFactory;