'use strict';

var ViewDependency = require('./ViewDependency');
var ContextDependencyHelpers = require('./ContextDependencyHelpers');
var RequireContextDependency = require('./RequireContextDependency');

class ViewDependencyParserPlugin {
  constructor(options) {
    this.options = options;
  }

  apply(parser) {
    var options = this.options || {};

    parser.hooks.call.for('createView').tap('OpenUI5Plugin', function (expr) {
      var dep = void 0;
      var param0 = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        dep = new ViewDependency(param0.string, expr.arguments[1].name, expr.range);
        if (!dep) {
          return false;
        }
      } else if (expr.arguments[0].type === 'Identifier') {
        dep = ContextDependencyHelpers.createView(RequireContextDependency, expr.range, param0, expr, options);
      } else {
        return false;
      }
      dep.loc = expr.loc;
      dep.optional = !!parser.scope.inTry;
      parser.state.current.addDependency(dep);
      return true;
    });
  }
}

module.exports = ViewDependencyParserPlugin;